; serial_receive_silabs.a51 	receives a single byte from pc using UART; shows it on display (its ascii value)
; June 7, 2010    Also echoes character, sending it back to PC
	
; rev path 3/16
	$NOSYMBOLS			; keeps listing short
	$INCLUDE (C:\SiLabs\MCU\INC\c8051f410.inc) 
	$INCLUDE (C:\SiLabs\MCU\INC\VECTORS320.INC)	; Tom's vectors definition file

	STACKBOT EQU 80h					; put stack at start of scratch indirectly-addressable block (80h and up)

;	TIMER96K EQU	0FDh	; 	timer value for 9600 baud (Dallas, p. 296 table)
	TIMER96K EQU 96h		; timer reload value---SiLabs, 24MHz div-by-12	

	SAY EQU 00h				; command says next characters (till ;) are speech codes
	EOM EQU ';'   			; EOM for ascii mode

	LF EQU 0Ah				; linefeed

	DISPLAY EQU P1
	KEYPAD EQU P2
	
	
	START_OFFSET EQU 01h	; initial offset into speech table (may have to tinker, to get it right)

	ORG 0h
	LJMP STARTUP

	ORG 080h
				
STARTUP: MOV SP, #STACKBOT-1

	acall USUAL_SETUP
	acall IO_SETUP
	acall TIMER_SETUP
	acall SERIAL_SETUP

;----receive code

AWAIT:	jnb RI0, $	; await receipt of a byte from UART
	clr RI0			; ...clear flag when it's asserted
	mov DISPLAY, SBUF0	; get received data
  sjmp AWAIT	; show each character that comes in, forever
	


;* * * * * * * * * * * * *	Test Code to exercise the serial port


;------- INITIALIZATIONS

; set up timer 1
TIMER_SETUP:    MOV TMOD, #020h		; Timer 1: Mode 2 (8-bit reload) (Dallas sample pgm, p. 299)
    MOV TH1, #TIMER96K	; Reload for 9600, if SMOD = 0 (table, p. 296, again)
    MOV TL1, #TIMER96K	; This is for first-pass only
    
    ANL (PCON), #07Fh	; clear SMOD bit (this is default, but let's be thorough): 
    							;  prevents baud-rate doubling at Serial Port 0 (p. 21)
	ret
		
	;Set up Serial Port 0
SERIAL_SETUP:
;	MOV	SCON0, #050h	; Mode 1, set receive enable (pp. 29-30).  Sample pgm calls this "SCON"
	mov SCON0, #10h			; set receive enable, 8-bit rather than "9-bit" or what the Dallas pgm calls 10-bit
;	SETB	ES0 				;	Enable Serial Port interrupt
;	SETB	EA 				;	Set Global Interrupt enable
	SETB	TR1 				;	Start Timer
	ret



USUAL_SETUP: anl   PCA0MD, #NOT(040h)        ; Disable the WDT.
                                             ; Clear Watchdog Enable bit

				; Configure the Oscillator
 ;            orl   OSCICN, #04h         ; sysclk = 24.5 Mhz / 8
			mov OSCICN, #087h	; sysclk = 24.5 MHz (full-speed)

         ; Enable the Port I/O Crossbar

            mov   XBR1, #40h           ; Enable Crossbar
						ret

IO_SETUP: orl P0MDOUT, #10h								; enable UTX as push-pull output
		mov XBR0, #01h						; enable UART on P0.4 (tx) and P0.5 (rx)
		ret

	END							
	
								
									
